unit Pintfm;

interface

uses
  Windows, SysUtils, Classes, Forms, Dialogs,
  Buttons, Controls, StdCtrls, Grids, ExtCtrls, PintCalcs;

const kscroll = 8;

type
  TPintForm = class(TForm)
    FindInFileDialog: TOpenDialog;
    BitBtn1: TBitBtn;
    OutNameEdit: TEdit;
    Number1Vars: TEdit;
    OutFileLabel: TLabel;
    Nvar1Label: TLabel;
    NumberFixedEffects: TEdit;
    NFixedLabel: TLabel;
    NVar2Label: TLabel;
    Number2Vars: TEdit;
    OutFileLabel2: TLabel;
    InFileLabel: TLabel;
    InNameEdit: TEdit;
    Explanation: TMemo;
    PathNameLabel: TLabel;
    PathNameEdit: TEdit;
    ReadyLabel: TLabel;
    FinalChoice: TRadioGroup;
    FinalChoiceBitBtn1: TBitBtn;
    FinalChoiceBitBtn0: TBitBtn;
    FinalChoiceBitBtn2: TBitBtn;
    procedure FormActivate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure AskParameters(Sender: TObject);
    procedure EnterCheckNumberVars(Sender: TObject; var Key: Char);
    procedure CheckNumber1Vars(Sender: TObject);
    procedure CheckNumberFixed(Sender: TObject);
    procedure CheckNumber2Vars(Sender: TObject);
    procedure EnterCheckNumberFixed(Sender: TObject; var Key: Char);
    procedure EnterCheckNumber2Vars(Sender: TObject; var Key: Char);
    procedure PlaceOKButton(Sender: TObject);
    procedure Calculate(Sender: TObject);
    procedure SaveOnly(Sender: TObject);
    procedure NoAction(Sender: TObject);
  private
    { Private declarations }
    procedure WriteParameterFile;
  public
    { Public declarations }
  end;


  mat = array[1..dimmax,1..dimmax] of real;
  vec = array[1..dimmax] of real;
var
  PintForm: TPintForm;
  inf: textfile;
  fn1, fn2, infn, outfn, outrfn, pathname, si: shortstring;
  error, warned, extension, BudgetConstraint, StopIt: boolean;
  ch: Char;

  k, k1, k2, kw, kb, c, nmin, nstep, nmax, n2min, n2max, bud: integer;
(*
 k = number of level-1 variables;
 k1 = number of level-1 variables with fixed effect only;
   (the other have zero within-group means)
 kw = number of level-2 variables;
 kb = k1+kw;
 k2 = number of random effects at level 2;
*)
  sigma2: real;
  mu1, mu3: vec;
  tau2, sigw, sigb: mat;

procedure ReadParameterFile;

implementation

uses CheckAnswer, AmountUnit, FillUnit;

{$R *.DFM}

procedure TPintForm.FormActivate(Sender: TObject);
var i,j: integer;
  cdir: string;
begin
  error := false;
  k := 0; k1 := 0; kw := 0;
  nmin := 10; nstep := 5; nmax := 50;
  c := 5;
  n2min := 10; n2max := 50;
  bud := 50*(c + 25);
  sigma2 := 1.0;
  for i:=1 to dimmax do begin
    for j:=1 to dimmax do begin
      tau2[i,j] := 0.0;
      sigw[i,j] := 0.0;
      sigb[i,j] := 0.0;
     end;
    tau2[i,i] := 1.0;
    sigw[i,i] := 1.0;
    sigb[i,i] := 1.0;
    mu1[i] := 0.0;
    mu3[i] := 1.0;
   end;
  BudgetConstraint := true;

  infn := '';
  with FindInFileDialog do begin
    if not Execute then begin
      infn := 'ax.dat';
     end else infn := FileName;
   end;
  fn1 := ExtractFileName(infn);
  pathname := ExtractFilePath(ExpandFileName(infn));
  outrfn := '';
  extension := false;
  for i:= 1 to length(fn1) do begin
     ch := fn1[i];
     if (ch = '.') then extension := true;
     if not extension then outrfn := outrfn+ch;
    end;
  InNameEdit.text := fn1;
  InNameEdit.ReadOnly := true;
  OutNameEdit.Text := outrfn+'.out';
  PathNameEdit.text := pathname;
  cdir := ExtractFileDir(ExpandFileName(infn));
  if SetCurrentDir(cdir) then ; (* seems not to work *)

  if fileexists(infn) then begin assignfile(inf,infn);
    reset(inf);
    ReadParameterFile;
    closefile(inf);
   end;

  Number1Vars.Text := inttostr(k);
  NumberFixedEffects.Text := inttostr(k1);
  Number2Vars.Text := inttostr(kw);
  explanation.lines[0] := 'Give output filename to be written by PINT';
  explanation.lines[1] := 'and the numbers of variables.';
  Application.ProcessMessages;

end;

procedure TPintForm.FormClose(Sender: TObject; var Action: TCloseAction);

begin
  Action := caFree;
end;

procedure TPintForm.AskParameters(Sender: TObject);
var x: real;
 ix,i,j: integer;

begin
  if error then begin
   close;
   exit;
  end;

  repeat
   warned := false;
   outfn := OutNameEdit.Text;
   if (outfn = '') then outfn := 'pint.out';
   if fileexists(outfn) then with ExistsForm do begin
     WarnLabel1.caption := 'Warning: file '+outfn+' is an existing file.';
     ShowModal;
     if (ModalResult = mrCancel) then exit;
    end;

   fn1 := ExtractFileName(outfn);
   outrfn := '';
   extension := false;
   for i:= 1 to length(fn1) do begin
      ch := fn1[i];
      if (ch = '.') then extension := true;
      if not extension then outrfn := outrfn+ch;
     end;

    ExistsForm.WarnLabel1.caption := '';
    fn2 := pathname+outrfn+'.cov';
 (* Check whether a file with name fn2 exists,
    if yes, replace extension .COV by CO1, or CO2, etc. up to CO9: *)

    i := 0;
    repeat
      inc(i);
      str(i,si);
      if fileexists(fn2) then begin
        fn2 := pathname+outrfn+'.co'+si;
        if (i = 9) then begin
        if fileexists(fn2) then begin
          ExistsForm.WarnLabel1.caption := 'Warning: file '+fn2+' is an existing file.';
          explanation.lines[2] := 'Warning: existing file '+fn2;
          explanation.lines[3] := 'will be overwritten if you continue.';
          warned := true;
         end;
        end;
       end;
     until ((not fileexists(fn2)) or (i >= 9));

   if warned then begin
     if ExistsForm.Showmodal = mrOK then warned := false;
     for i:=0 to 3 do explanation.lines[i] := '';
     explanation.lines[0] := 'Give parameter file to be written or used by PINT';
    end;
   if warned then exit;
  until (not warned);

  for i:=0 to 3 do explanation.lines[i] := '';
  Application.ProcessMessages;

  CheckNumber1Vars(Sender);
  CheckNumberFixed(Sender);
  CheckNumber2Vars(Sender);
  BitBtn1.visible := false;

  if (k1 >= 1) then with FillForm do begin
    if (k1 = 1) then begin
      explanation.lines[0] := 'Mean of the level-1 variable';
      FillFormLabel.caption := 'Fill in the mean of the level-1 variable';
     end else begin
      explanation.lines[0] := 'Means of the level-1 variables';
      FillFormLabel.caption := 'Fill in the means of the level-1 variables';
     end;
    Application.ProcessMessages;
    FillFormLabel2.caption := '  ';
    halfsymmetric := false;
    with StringGrid1 do begin
      rowcount := k1;
      colcount := 1;
      if (k1 >= kscroll) then ScrollBars := ssBoth else ScrollBars := ssNone;
      height := DefaultRowHeight * k1 + 15;
      width := DefaultColWidth + 15;
      for i:=1 to k1 do begin
        cells[0,i-1] := floattostr(mu1[i]);
       end;
     end; (* with StringGrid1 *)
    repeat showmodal until not FillFormError;
        (* hitting the OK button activates ReadValues *)
    for i:=1 to k1 do mu1[i] := a[i,1];
   end; (* with FillForm *)

  if (kw >= 1) then with FillForm do begin
    if (kw = 1) then begin
      explanation.lines[0] := 'Mean of the level-2 variable';
      FillFormLabel.caption := 'Fill in the mean of the level-2 variable';
     end else begin
      explanation.lines[0] := 'Means of the level-2 variables';
      FillFormLabel.caption := 'Fill in the means of the level-2 variables';
     end;
    Application.ProcessMessages;
    FillFormLabel2.caption := '  ';
    halfsymmetric := false;
    with StringGrid1 do begin
      rowcount := kw;
      colcount := 1;
      if (kw >= kscroll) then ScrollBars := ssBoth else ScrollBars := ssNone;
      height := DefaultRowHeight * kw + 15;
      width := DefaultColWidth + 15;
      for i:=1 to kw do begin
        cells[0,i-1] := floattostr(mu3[i]);
       end;
     end; (* with StringGrid1 *)
    repeat showmodal until not FillFormError;
    for i:=1 to kw do mu3[i] := a[i,1];
   end; (* with FillForm *)

  if (k >= 1) then with FillForm do begin
    if (k = 1) then begin
   FillFormLabel.caption := 'Fill in the within-groups variance of the level-1 variable';
   explanation.lines[0] := 'Within-groups variance ';
   explanation.lines[1] := 'of the level-1 variable';
    end else begin
   explanation.lines[0] := 'Within-groups covariance matrix';
   explanation.lines[1] := 'of the level-1 variables';
   FillFormLabel.caption := 'Fill in the within-groups covariance matrix (subdiagonal)';
   FillFormLabel2.caption := 'of the level-1 variables';
   if ((k1 > 0) and (k1 < k)) then begin
     Explanation.lines[2] := 'Level-1 variables must be ordered as follows:';
     if (k1 = 1) then
       Explanation.lines[3] := 'first the variable with a fixed effect only,'
      else
       Explanation.lines[3] := 'first variables with fixed effects only,';
     if (k1 = k-1) then
       Explanation.lines[4] := 'then the variable with fixed as well as random effect.'
      else
       Explanation.lines[4] := 'then variables with fixed as well as random effects.';
    end;

   Application.ProcessMessages;

   with StringGrid1 do begin
     rowcount := k;
     colcount := k;
     Height := DefaultRowHeight * k + 15;
     Width := DefaultColWidth * k + 15;
     halfsymmetric := true;
     if (k >= kscroll) then ScrollBars := ssBoth else ScrollBars := ssNone;
     for i:=1 to k do begin
       for j:=1 to i-1 do cells[j-1,i-1] := floattostr(sigw[i,j]);
       cells[i-1,i-1] := floattostr(sigw[i,i]);
       for j:=i+1 to k do cells[j-1,i-1] := ' ';
      end;
    end; (* with StringGrid1 *)
   repeat showmodal until not FillFormError;
   for i:=1 to k do for j:=1 to k do
     if (i >= j) then sigw[i,j] := a[i,j] else sigw[i,j] := a[j,i];
   end; (* with FillForm *)
  end; (* if k >= 1 *)

   for i:=0 to 3 do explanation.lines[i] := '';
   kb := k1+kw;
   if (kb > 0) then begin
     if (kb = 1) then begin
       explanation.lines[0] := 'Between-groups variance ';
       explanation.lines[1] := 'of explanatory variable';
      end else begin
       explanation.lines[0] := 'Between-groups covariance matrix';
       explanation.lines[1] := 'of explanatory variables';
      end;
    if (k1 = 1) then begin
      explanation.lines[2] := '(The group mean of the level-1 variable';
      explanation.lines[3] := 'is treated as a level-2 variable.)';
     end;
    if (k1 >= 2) then begin
      explanation.lines[2] := '(The group means of the level-1 variables';
      explanation.lines[3] := 'are treated as level-2 variables.)';
     end;
    if (kb = 1) then begin
      explanation.lines[5] := 'There is in total one variable at level 2.';
     end else begin
      explanation.lines[5] := 'There are in total '+inttostr(k1+kw)+' variables at level 2.';
     end;
    if ((kw > 0) and (k1 > 0)) then begin
      explanation.lines[6] := 'They must be ordered so ';
      if (kw = 1) then
        explanation.lines[7] := 'that first comes the original level-2 variable,'
       else
        explanation.lines[7] := 'that first come the '+inttostr(kw)+' original level-2 variables,';
      if (k1 = 1) then begin
        explanation.lines[8] := 'and then the group mean of the';
        explanation.lines[9] := 'of the level-1 variable with fixed effects only.';
       end else begin
        explanation.lines[8] := 'and then the group means of the';
        explanation.lines[9] := 'of the level-1 variables with fixed effects only.';
       end;
     end; (* if ((kw > 0) and (k1 > 0))  *)
    Application.ProcessMessages;

    with FillForm do begin
      if (kb = 1) then
        FillFormLabel.caption := 'Fill in the between-groups variance'
       else
        FillFormLabel.caption := 'Fill in the between-groups covariance matrix';
      with StringGrid1 do begin
        rowcount := kb;
        colcount := kb;
        if (kb >= kscroll) then ScrollBars := ssBoth else ScrollBars := ssNone;
        Height := DefaultRowHeight * kb + 15;
        Width := DefaultColWidth * kb + 15;
        halfsymmetric := true;
        for i:=1 to kb do begin
          for j:=1 to i-1 do cells[j-1,i-1] := floattostr(sigb[i,j]);
          cells[i-1,i-1] := floattostr(sigb[i,i]);
          for j:=i+1 to kb do cells[j-1,i-1] := ' ';
         end;
       end; (* with StringGrid1 *)
      repeat showmodal until not FillFormError;
      for i:=1 to kb do for j:=1 to kb do
        if (i >= j) then sigb[i,j] := a[i,j] else sigb[i,j] := a[j,i];
     end; (* with FillForm *)
   end; (* if kb > 0 *)

   for i:=0 to 9 do explanation.lines[i] := '';
   explanation.lines[0] := 'Parameters of the random part of the model.';
   explanation.lines[1] := '1. Residual variance at level 1.';
   Application.ProcessMessages;

   with FillForm do begin
    FillFormLabel.caption := 'Fill in the level-1 residual variance';
    FillFormLabel2.caption := '  ';
    halfsymmetric := false;
    with StringGrid1 do begin
      rowcount := 1;
      colcount := 1;
      ScrollBars := ssNone;
      height := DefaultRowHeight + 15;
      width := DefaultColWidth + 15;
      cells[0,0] := floattostr(sigma2);
     end; (* with StringGrid1 *)
    repeat showmodal until not FillFormError;
        (* hitting the OK button activates ReadValues *)
    sigma2 := a[1,1];
    if (sigma2 <= 0.0) then sigma2 := 0.000001;
   end; (* with FillForm *)

   k2 := k-k1+1;
   if (k2 = 1) then begin
     explanation.lines[1] := '2. Intercept variance at level 2.';
     explanation.lines[3] := 'The model contains a random intercept and no random slopes.';
     explanation.lines[4] := 'The intercept variance is needed.';
    end else begin
     explanation.lines[1] := '2. Covariance matrix of random effects at level 2.';
     explanation.lines[3] := 'There are '+inttostr(k2)+' random coefficients (intercept plus '
                               +inttostr(k2-1)+' slopes).';
     explanation.lines[4] := 'Their covariance matrix is required.';
    end;
   Application.ProcessMessages;


    with FillForm do begin
      if (k2 = 1) then
        FillFormLabel.caption := 'Fill in the intercept variance'
       else
        FillFormLabel.caption := 'Fill in the level-two random effects covariance matrix';
      with StringGrid1 do begin
        rowcount := k2;
        colcount := k2;
        if (k2 >= kscroll) then ScrollBars := ssBoth else ScrollBars := ssNone;
        Height := DefaultRowHeight * k2 + 15;
        Width := DefaultColWidth * k2 + 15;
        halfsymmetric := true;
        for i:=1 to k2 do begin
          for j:=1 to i-1 do cells[j-1,i-1] := floattostr(tau2[i,j]);
          cells[i-1,i-1] := floattostr(tau2[i,i]);
          for j:=i+1 to k2 do cells[j-1,i-1] := ' ';
         end;
       end; (* with StringGrid1 *)
      repeat showmodal until not FillFormError;
      for i:=1 to k2 do for j:=1 to k2 do
        if (i >= j) then tau2[i,j] := a[i,j] else tau2[i,j] := a[j,i];
     end; (* with FillForm *)

   for i:=0 to 9 do explanation.lines[i] := '';
   explanation.lines[0] := 'Determination of the amount of output.';
   explanation.lines[1] := '';
   explanation.lines[2] := 'There are two options for the amount of output:';
   explanation.lines[3] := '1. Sample sizes must satisfy a linear budget constraint';
   explanation.lines[4] := '2. All combinations of sample sizes are given';
   explanation.lines[5] := '   up to maxima specified by the user.';
   explanation.lines[6] := '(option 2 gives more output)';
   Application.ProcessMessages;

   with AmountForm do begin
     if BudgetConstraint then begin
       OutputSelection.ItemIndex := 0;
       RangeParameters.visible := false;
       BudgetParameters.visible := true;
      end else begin
       OutputSelection.ItemIndex := 1;
       RangeParameters.visible := true;
       BudgetParameters.visible := false;
      end;
     SetAmounts;
     repeat showmodal until not AmountFormError;
    end;

  for i:=0 to 9 do explanation.lines[i] := '';
  explanation.lines[0] := 'All parameter values now have been determined.';
  explanation.lines[2] := 'Continue by choosing what to do with them.';
  ReadyLabel.visible := true;
  FinalChoiceBitBtn0.visible := true;
  FinalChoice.visible := true;
end;


procedure TPintForm.EnterCheckNumberVars(Sender: TObject; var Key: Char);
begin
  if (ord(key) = 13) then CheckNumber1Vars(Sender);
end;

procedure TPintForm.CheckNumber1Vars(Sender: TObject);
var i: integer;
begin
  try
    k := strtoint(Number1Vars.text);
    except on E: EConvertError do k := 0;
   end; (* try *)
  if (k < 0) then begin
    k := 0;
    Number1Vars.text := '0';
   end;
  if (k > dimmax -1) then begin
    k := dimmax -1;
    Number1Vars.text := inttostr(dimmax-1);
    for i:=0 to 9 do explanation.lines[i] := '';
    explanation.lines[0] := 'warning: ';
    explanation.lines[2] := 'number of level-1 variables is bounded';
    explanation.lines[3] := 'by the restriction ';
    explanation.lines[4] := 'k1 + (k-k1+1)*(kw+1) <= dimmax';
    explanation.lines[5] := 'where k = # level-1 vars.,';
    explanation.lines[6] := 'where k1 = # level-1 vars. with fixed effects,';
    explanation.lines[7] := 'where kw = # level-2 vars.,';
    explanation.lines[8] := 'dimmax = '+inttostr(dimmax)+' is a constant in the program.';
    Application.ProcessMessages;
   end else begin
    for i:=0 to 9 do explanation.lines[i] := '';
    explanation.lines[0] := 'Give output filename to be written by PINT';
    explanation.lines[1] := 'and the numbers of variables.';
    Application.ProcessMessages;
   end;
end;

procedure TPintForm.CheckNumberFixed(Sender: TObject);
var i: integer;
begin
  CheckNumber1Vars(Sender);
  try
    k1 := strtoint(NumberFixedEffects.text);
    except on E: EConvertError do k1 := 0;
   end; (* try *)
  if (k1 < 0) then begin
    k1 := 0;
    NumberFixedEffects.text := '0';
   end;
  if (k1 > k) then begin
    k1 := k;
    NumberFixedEffects.text := inttostr(k1);
    for i:=0 to 9 do explanation.lines[i] := '';
    explanation.lines[0] := 'warning: ';
    explanation.lines[2] := 'number of variables with fixed effects';
    explanation.lines[3] := 'cannot be larger than number level-1 variables.';
    Application.ProcessMessages;
   end else begin
    for i:=0 to 9 do explanation.lines[i] := '';
    explanation.lines[0] := 'Give output filename to be written by PINT';
    explanation.lines[1] := 'and the numbers of variables.';        
    Application.ProcessMessages;
   end;
end;

procedure TPintForm.CheckNumber2Vars(Sender: TObject);
var i: integer;
begin
  CheckNumber1Vars(Sender);
  try
    kw := strtoint(Number2Vars.text);
    except on E: EConvertError do kw := 0;
   end; (* try *)
  if (kw < 0) then begin
    kw := 0;
    Number2Vars.text := '0';
   end;
  if (kw > dimmax-1-k) then begin
    kw := dimmax-1-k;
    Number2Vars.text := inttostr(kw);
    for i:=0 to 9 do explanation.lines[i] := '';
    explanation.lines[0] := 'warning: ';
    explanation.lines[2] := 'number of level-2 variables is bounded';
    explanation.lines[3] := 'by the restriction ';
    explanation.lines[4] := 'k1 + (k-k1+1)*(kw+1) <= dimmax';
    explanation.lines[5] := 'where k = # level-1 vars.,';
    explanation.lines[6] := 'where k1 = # level-1 vars. with fixed effects,';
    explanation.lines[7] := 'where kw = # level-2 vars.,';
    explanation.lines[8] := 'dimmax = '+inttostr(dimmax)+' is a constant in the program.';
    Application.ProcessMessages;      
   end else begin
    for i:=0 to 9 do explanation.lines[i] := '';
    explanation.lines[0] := 'Give output filename to be written by PINT';
    explanation.lines[1] := 'and the numbers of variables.';
    Application.ProcessMessages;
   end;                         
end;

procedure TPintForm.EnterCheckNumberFixed(Sender: TObject; var Key: Char);
begin
  if (ord(key) = 13) then CheckNumberFixed(Sender);
end;

procedure TPintForm.EnterCheckNumber2Vars(Sender: TObject; var Key: Char);
begin
  if (ord(key) = 13) then CheckNumber2Vars(Sender);
end;



PROCEDURE ReadParameterFile;

(*
 k = number of level-1 variables;
 k1 = number of level-1 variables with fixed effect only;
   (the other have zero within-group means)
 kw = number of level-2 variables;
 kb = k1+kw;
 k2 = number of random effects at level 2;
  sigma2: real;
  mu1, mu3: vec;
  tau2, sigw, sigb: mat;
*)
VAR i, j, q :INTEGER;
(* OP INPUT STAAN RESP.
   k : AANTAL VARIABELEN OP LL. NIVEAU INCL. CONSTANTE
   k1    : AANTAL VARIABELEN MET VASTE EFFECTEN OP LL.NIVEAU
   kw    : AANTAL VARIABELEN OP SCHOOLNIVEAU INC. CONSTANTE
   nmin: aantal leerlingen per school waarmee we beginnen
   nstep : de stapgrootte (aantal leerlingen dat per school toegvoegd
          wordt voor de volgende berekening begint)
   nmax : maximum aantal leerlingen per school
   bud:in een nader te bepalen eenheid uit te drukken totale
          beschikbare gelden voor dataverzameling; uitgangspunt is dat de
          eenheid is uitgedrukt als: wat kost het om 1 leerling te
          onderzoeken
   c :de extra kosten die verbonden zijn aan het samplen van een
          nieuwe school (in dezelfde eenheid als 'kosten' uit te drukken)
  sigma2 : RESIDUELE VARIANTIE IN HET MODEL
  tau2    : onderdriehoekscovariantiematrix van de variantiecomponenten
  sigw  : covariantiematrix van de leerlingvariabelen
            eerst vaste EFFEcten, dan constante, dan random EFFEcten
  sigb  : covariantiematrix van de schoolvariabelen
            eerst constante, dan echte schoolvariabele, dan
            schoolgemiddelde van de leerlingvariabele met vaste EFFEcten
  mu1     : gemiddelden van de schoolgemiddelden der leerlingvariabelen met
            vaste EFFEcten
  mu3     : gemiddelden van de variabelen behorend bij sigma(s) m.u.v.
            schoolgemiddelden van leerlingvariabelen die zitten in mu1
  *)

BEGIN
(* First the input without the effects of the constant *)
 StopIt := false;
 read(inf, k , k1 , kw);
 q := k - k1 ;
 IF (k1+(k-k1+1)*(kw+1) > dimmax) THEN BEGIN
   StopIt := true;
   exit;
  END;
 if eof(inf) then exit;
 read(inf, nmin, nstep, nmax);
 if (nstep >= 0) then begin
    read(inf, bud , c );
    BudgetConstraint := true;
   end else begin
    read(inf, n2min, n2max);
    BudgetConstraint := false;
    nstep := -nstep;
   end;
 if eof(inf) then exit;
 read(inf,sigma2);
 FOR i := 1 TO q+1 DO
  FOR j := 1 TO i DO
   BEGIN
    read(inf,tau2[i,j]);
    tau2[j,i] := tau2[i,j];
   END;
 if eof(inf) then exit;
 FOR i := 1 TO k1+q DO BEGIN
  FOR j := 1 TO i DO
   read(inf,sigw[i,j]); readln(inf); END;
 if eof(inf) then exit;
 FOR i := 1 TO kw+k1   DO BEGIN
  FOR j := 1 TO i   DO
   read(inf,sigb[i,j]); readln(inf); END;
 if eof(inf) then exit;
 FOR i := 1 TO k1 DO
  read(inf,mu1[i]);
 if eof(inf) then exit;
 FOR i := 1 TO kw DO
  read(inf,mu3[i]);
END; (* ReadParameterFile *)


procedure TPintForm.PlaceOKButton(Sender: TObject);
begin
  case FinalChoice.ItemIndex of
    0: begin
         FinalChoiceBitBtn0.Visible := true;
         FinalChoiceBitBtn1.Visible := false;
         FinalChoiceBitBtn2.Visible := false;
       end;                            
    1: begin
         FinalChoiceBitBtn0.Visible := false;
         FinalChoiceBitBtn1.Visible := true;
         FinalChoiceBitBtn2.Visible := false;
       end;
    2: begin
         FinalChoiceBitBtn0.Visible := false;
         FinalChoiceBitBtn1.Visible := false;
         FinalChoiceBitBtn2.Visible := true;
       end;
  end; (* case *)
end;


procedure TPintForm.WriteParameterFile;
var i,j: integer;
begin
  assignfile(inf,infn);
  rewrite(inf);
  writeln(inf,k:6,' ',k1:6,' ',kw:6);
  if BudgetConstraint then begin
    writeln(inf,nmin:6,' ',nstep:6,' ',nmax:6);
    writeln(inf,bud:6,' ',c:6);
   end else begin
    writeln(inf,nmin:6,' ',-1*nstep:6,' ',nmax:6);
    writeln(inf,n2min:6,' ',n2max:6);
   end;
  writeln(inf,sigma2:20:10);
  for i:=1 to k2 do begin
    for j:=1 to i do write(inf,tau2[i,j]:15:7,' '); writeln(inf);
   end;
  for i:=1 to k do begin
    for j:=1 to i do write(inf,sigw[i,j]:15:7,' '); writeln(inf);
   end;
  for i:=1 to kb do begin
    for j:=1 to i do write(inf,sigb[i,j]:15:7,' '); writeln(inf);
   end;
  for i:=1 to k1 do write(inf,mu1[i]:15:7,' '); writeln(inf);
  for i:=1 to kw do write(inf,mu3[i]:15:7,' '); writeln(inf);
  writeln(inf);
  writeln(inf,'Results written to files ',outfn,' and ',fn2,'.');
  closefile(inf);
end; (* WriteParameterFile *)

procedure TPintForm.Calculate(Sender: TObject);
begin
  WriteParameterFile;
  CalculateStandardErrors(infn, outfn, fn2);
  close;
end;

procedure TPintForm.SaveOnly(Sender: TObject);
begin
  WriteParameterFile;
  close;
end;

procedure TPintForm.NoAction(Sender: TObject);
begin
  close;
end;

begin
  ForceCurrentDirectory := true;
end.
